{% extends "spider_base/base.html" %}

{% load spider_rdf %}

{% block public_meta %}
  {{block.super}}
  {% if scope == "list" %}
    <meta name="description" content="{{uc.description|escape}}"/>
  {% else %}
    <meta name="description" content="{{object.localized_description|escape}}"/>
  {% endif %}
{% endblock %}


{% block context_actions %}
{{block.super}}
{% endblock context_actions %}

{# don't include color information here; they could leak the strength  #}
{% block main_classes %}{{block.super}} spkc-content w3-card-4{% endblock %}
{% block main_attributes %}{{block.super}} prefix="spkc: {% spkc_namespace %} xsd: http://www.w3.org/2001/XMLSchema# rdf: http://www.w3.org/1999/02/22-rdf-syntax-ns#" resource="{{hostpart}}{{ request.path }}"{% endblock %}
