# Generated by Django 2.1 on 2018-08-20 14:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0001_initial'),
        ('spider_filets', '0002_textfilet_non_public_edit'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='textfilet',
            name='non_public_edit',
        ),
        migrations.AddField(
            model_name='textfilet',
            name='editable_from',
            field=models.ManyToManyField(help_text='Allow editing from selected componentsby privileged users.', related_name='_textfilet_editable_from_+', to='spider_base.UserComponent'),
        ),
    ]
