# Generated by Django 2.1 on 2018-08-15 12:05

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spider_tags', '0002_auto_20180815_1128'),
    ]

    operations = [
        migrations.AlterField(
            model_name='spidertag',
            name='primary',
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AlterField(
            model_name='spidertag',
            name='tagdata',
            field=jsonfield.fields.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='spidertag',
            name='verified_by',
            field=jsonfield.fields.JSONField(blank=True, default=list),
        ),
    ]
