# Generated by Django 2.1.7 on 2019-02-26 18:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0002_20190518_squashed'),
        ('spider_tags', '0005_spidertag_updateable_by'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='spidertag',
            name='updateable_by',
        ),
        migrations.AddField(
            model_name='spidertag',
            name='updateable_by',
            field=models.ManyToManyField(blank=True, related_name='tags', to='spider_base.ReferrerObject'),
        ),
    ]
