# Generated by Django 2.1.5 on 2019-02-11 20:15

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spider_tags', '0004_auto_20190117_1253'),
    ]

    operations = [
        migrations.AddField(
            model_name='spidertag',
            name='updateable_by',
            field=jsonfield.fields.JSONField(blank=True, default=list),
        ),
    ]
