# Generated by Django 2.2.3 on 2019-07-08 13:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_webcfg', '0002_auto_20190120_1252'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webconfig',
            name='config',
            field=models.BinaryField(blank=True, default=b'', editable=True),
        ),
    ]
