{% load i18n spider_base %}
<data hidden="hidden" rel="spkc:properties">
  <data hidden="hidden" typeof="spkc:Property">
    <data hidden="hidden" property="spkc:name" datatype="xsd:string">name</data>
    <data hidden="hidden" property="spkc:value" datatype="xsd:string">{{object}}</data>
    <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">true</data>
  </data>
</data>
<div class="w3-padding">
  <div class="w3-white w3-padding" typeof="spkc:Property">
    <data hidden="hidden" property="spkc:name" datatype="xsd:string">text</data>
    <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">true</data>
    <data property="spkc:value" datatype="xsd:string">{{object.text|safe}}</data>
  </div>
</div>
{% include "spider_filets/license_info.html" %}
{% active_feature_names as fnames %}
{% if "EUDisclaimer" in fnames %}
<div class="w3-padding w3-small w3-text-red">
  {% blocktrans %}
  The author maybe profits financially from this blog post.
  {% endblocktrans %}
</div>
{% endif %}
{% if "update_guest" in abilities %}
<div class="w3-padding">
  <a href="{% url 'spider_base:ucontent-access' token=object.associated.token access='update_guest' %}?{{sanitized_GET}}">{% trans "Update" %}</a>
</div>
{% endif %}
