{% for name, value in widget.wrapper_attrs.items %}{% if value is not False and name != skipname %} {{ name }}{% if value is not True %}="{{ value|stringformat:'s' }}"{% endif %}{% endif %}{% endfor %}
