{% extends "registration/base.html" %}
{% load i18n static %}

{% block content %}
  {% trans 'Update User Profile' as legend %}
  {% trans 'Update' as confirm %}
  {% include "spider_base/edit_form.html" with form=form legend=legend confirm=confirm %}
{% endblock %}
