# Generated by Django 2.2.2 on 2019-06-21 20:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_keys', '0003_auto_20190621_2012'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='anchorkey',
            name='key',
        ),
    ]
