{% load spider_rdf %}
{% include "spider_base/partials/form_errors.html" with form=form %}
<{% if not inner_form %}form action="{{ request.get_full_path }}" enctype="{{enctype}}" method="post"{% else %}div{% endif %}>
  {% if not inner_form %}<input type="hidden" property="spkc:csrftoken" name="csrfmiddlewaretoken" datatype="xsd:string" content="{{csrf_token}}" value="{{csrf_token}}">{% endif %}
  {# fieldsets don't allow horizontal scrolling #}
  <h2>{{legend|safe}}</h2>
  <div class="w3-border w3-padding">
    {% include "spider_base/partials/base_form.html" with form=form %}
  </div>
  {% if not inner_form %}
    <button class="w3-button w3-grey" style="margin-bottom: 5px">
      {{confirm}}
    </button>
  {% endif %}
<{% if not inner_form %}/form{% else %}/div{% endif %}>
