# Generated by Django 2.2 on 2019-04-19 08:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0011_auto_20190405_0045'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='filefilet',
            name='license',
        ),
        migrations.RemoveField(
            model_name='textfilet',
            name='license',
        ),
        migrations.AddField(
            model_name='filefilet',
            name='license_url',
            field=models.URLField(blank=True, max_length=400, null=True),
        ),
        migrations.AddField(
            model_name='textfilet',
            name='license_url',
            field=models.URLField(blank=True, max_length=400, null=True),
        ),
    ]
