{% load i18n %}

<div class="w3-padding">
  <h1>{% trans 'Key & hash' %}</h1>
  <table>
    <tr>
      <td>{% trans 'Hash algorithm' %}</td>
      <td class="w3-white w3-padding" typeof="spkc:Property">
        <data hidden="hidden" property="spkc:name" datatype="xsd:string">hash_algorithm</data>
        <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">false</data>
        <span property="spkc:value" datatype="xsd:string">{{hash_algo}}</span>
      </td>
    </tr>
    <tr>
      <td>{% trans 'Hash' %}</td>
      <td class="w3-white w3-padding" style="word-break: break-all" typeof="spkc:Property">
        <data hidden="hidden" property="spkc:name" datatype="xsd:string">hash</data>
        <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">false</data>
        <span property="spkc:value" datatype="xsd:string">{{hash}}</span>
      </td>
    </tr>
    {% if pubkeyhash %}
    <tr>
      <td>{% trans 'Hash of public key' %}</td>
      <td class="w3-white w3-padding" style="word-break: break-all" typeof="spkc:Property">
        <data hidden="hidden" property="spkc:name" datatype="xsd:string">pubkeyhash</data>
        <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">false</data>
        <span property="spkc:value" datatype="xsd:string">{{pubkeyhash}}</span>
      </td>
    </tr>
    {% endif %}
    <tr style="margin-top:5px">
      <td>{% trans 'key' %}</td>
      <td class="w3-white w3-padding" style="word-break: break-all" typeof="spkc:Property">
        <data hidden="hidden" property="spkc:name" datatype="xsd:string">key</data>
        <data hidden="hidden" property="spkc:hashable" datatype="xsd:boolean">true</data>
        <span property="spkc:value" datatype="xsd:string">{{object.key|linebreaksbr}}</span>
      </td>
    </tr>
  </table>
</div>
