{% load i18n %}
{% block errors %}
  {% if form.errors or form.non_field_errors %}
    <div class="w3-red w3-padding">
      <p class="">
        {% if form.errors.items|length == 1 %}{% trans "Please correct the error below." %}{% else %}{% trans "Please correct the errors below." %}{% endif %}
      </p>
      {% if form.non_field_errors %}
        <ul>
          {% for error in form.non_field_errors %}
          <li class="">
              {{ error }}
          </li>
          {% endfor %}
        </ul>
      {% endif %}
    </div>
  {% endif %}

{% endblock errors %}
